<TeXmacs|1.0.7.3>

<style|generic>

<\body>
  <hlink|<section|Aesthete Help>|index.html>

  <section|Glypher>

  <subsection|An equation editor / CAS frontend to SymPy>

  Glypher allows users to formulate basic symbolic algebra problems in a
  natural, visually clear manner and aims to provide the full functionality
  of SymPy interactively illustrated in pretty <hlink|Linux
  Libertine|http://www.linuxlibertine.org> fonts.

  To provide an efficient and powerful equation editor, the initial learning
  curve is necessarily a bit steep, but after a few examples it will all
  become familiar!

  Glypher is inspired by, and aims to maintain some usage compatibility with,
  the excellent scientific editor <hlink|TeXmacs|http://www.texmacs.org>.

  <subsubsection|Usage>

  Conceptually, Glypher represents equations as a combination of Entities,
  grouped in Phrases, and then into PhraseGroups. An Entity might be the
  letter <verbatim|c> or the vinculum of a fraction, a Phrase, the token
  <verbatim|cos> and a PhraseGroup, the combination of function name,
  bracketed Phrase and variable, <verbatim|cos(x)>. This hierarchy allows
  consistent and predictable rendering of Glypher equations into SymPy
  objects. Entities representing a unicode symbol are called Symbols. Note
  that this terminology contrasts with SymPy - a SymPy symbol is generated by
  a combination of one or more Glypher Symbols in a Phrase, allowing words to
  be used as SymPy symbols.

  One defining characteristic of Glypher is its insistence on handling
  bracketing internally. Brackets are implied, as each phrase will be treated
  distinctly. If two adjacent characters share a phrase, they should always
  have a green underline. Where brackets are required for clarity of reading,
  they are usually automatically inserted (and removed) by Glypher. For
  example, <verbatim|a*a^2> will be appear, rather than
  <verbatim|(a)*((a)^2)>.

  The cursor will allow you to attach to Symbols and Phrases. If you insert a
  PhraseGroup, such as an integral or binary expression, the active sites
  that it expects you to fill in, called TargetPhrases, will be highlighted
  with grey borders, shaded where they have not yet been filled-in by the
  user. To help you see the (major) PhraseGroup your cursor is located in, an
  orange bar appears along the top.

  To delete a PhraseGroup, you may need to empty some or all of the
  TargetPhrases first.

  Primarily for simplifying SymPy objects, Glypher performs a series of
  automatic simplifications, such as reducing nested (associative) binary
  expressions into a single n-ary expression, or cancelling double negatives
  in a summation.

  <subsubsection|Commands>

  To get Glypher to do anything useful, you need to write a command. Commands
  are split by spaces, normally forming a natural language sentence. For
  instance, <verbatim|Do> followed by a space and an expression is equivalent
  to calling SymPy's <verbatim|obj.doit()> on the equation's SymPy object.
  Unlike most n-ary expressions, you can usually press the spacebar from deep
  in a set of nested PhraseGroups and Glypher will assume you want to add a
  term to the top space-separated expression.

  <block|<tformat|<table|<row|<cell|Set A=EXP>|<cell|assign the expression on
  the right to every future occurance of A>>|<row|<cell|Unset A>|<cell|cancel
  the above>>|<row|<cell|Sub A=EXP1 into EXP2>|<cell|substitute A=EXP1 into
  EXP2>>|<row|<cell|Do A>|<cell|call S.doit() where S is A's SymPy
  object>>|<row|<cell|Expand EXP1>|<cell|call S.expand() where S is A's SymPy
  object>>|<row|<cell|Expand EXP1 about x=0>|<cell|do a Taylor expansion to
  order 4 of EXP1 (can add, e.g., `to order 5')>>|<row|<cell|Evaluate
  EXP1>|<cell|call S.evalf() with optional subsitutions>>|<row|<cell|Evaluate
  EXP1 at x=0,y=1>|<cell|>>|<row|<cell|Plot EXP1>|<cell|calls sympy's plot
  function>>|<row|<cell|Source EXP1>|<cell|creates a new Source for
  subsequent plotting in the Plot tab>>>>>

  To execute a command, simply press Return. The statement and response will
  be added to the list below and can be accessed subsequently through a
  reference of the form <verbatim|[s4]> or <verbatim|[r4]>, respectively.

  <subsubsection|Symbols & Alternatives>

  Many characters have one or more alternatives that can be accessed using
  Tab (or Shift+Tab, or by holding down Super, usu. Windows key, and pressing
  Up or Down) when attached to the Symbol. These are, in general, compatible
  with TeXmacs. All basic Greek characters are obtainable from the Latin
  characters.

  Some phrases may have alternatives, such as a product for a summation. This
  is indicated by a small, green triangle in the upper-left corner of the
  relevant Phrase. When attached, the keys above will let you move between
  the possibilities.

  <subsubsection|Pasting>

  Ctrl+Y will let you automatically <verbatim|sympify> and render the
  clipboard text into Glypher.

  <subsubsection|Keyboard shortcuts>

  <block|<tformat|<table|<row|<cell|Alt-F>|<cell|insert a
  fraction>>|<row|<cell|Alt-i>|<cell|insert an
  integral>>|<row|<cell|Alt-d>|<cell|insert a
  derivative>>|<row|<cell|^>|<cell|add superscript; use Ctrl+Super+Right to
  swap power/index>>|<row|<cell|_>|<cell|add subscript; use Ctrl+Super+Right
  to swap derivative/index>>|<row|<cell|Ctrl+(>|<cell|turn preceding word
  into a function>>|<row|<cell|Alt-c>|<cell|insert a
  summation>>|<row|<cell|Alt-[>|<cell|insert a reference to a previous
  statement or response>>>>>
</body>

<\references>
  <\collection>
    <associate|auto-1|<tuple|1|?>>
    <associate|auto-2|<tuple|2|?>>
    <associate|auto-3|<tuple|2.1|?>>
    <associate|auto-4|<tuple|2.1.1|?>>
    <associate|auto-5|<tuple|2.1.2|?>>
    <associate|auto-6|<tuple|2.1.3|?>>
    <associate|auto-7|<tuple|2.1.4|?>>
    <associate|auto-8|<tuple|2.1.5|?>>
  </collection>
</references>

<\auxiliary>
  <\collection>
    <\associate|toc>
      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Aesthete
      Help> <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-1><vspace|0.5fn>

      <vspace*|1fn><with|font-series|<quote|bold>|math-font-series|<quote|bold>|Glypher>
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-2><vspace|0.5fn>

      <with|par-left|<quote|1.5fn>|An equation editor / CAS frontend to SymPy
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-3>>

      <with|par-left|<quote|3fn>|Usage <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-4>>

      <with|par-left|<quote|3fn>|Commands
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-5>>

      <with|par-left|<quote|3fn>|Symbols & Alternatives
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-6>>

      <with|par-left|<quote|3fn>|Pasting <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-7>>

      <with|par-left|<quote|3fn>|Keyboard shortcuts
      <datoms|<macro|x|<repeat|<arg|x>|<with|font-series|medium|<with|font-size|1|<space|0.2fn>.<space|0.2fn>>>>>|<htab|5mm>>
      <no-break><pageref|auto-8>>
    </associate>
  </collection>
</auxiliary>